@@ -10,6 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from openstack.compute.v2 import aggregate as _aggregate
 from openstack.compute.v2 import availability_zone
 from openstack.compute.v2 import extension
 from openstack.compute.v2 import flavor as _flavor
@@ -18,6 +19,7 @@ from openstack.compute.v2 import image as _image
 from openstack.compute.v2 import keypair as _keypair
 from openstack.compute.v2 import limits
 from openstack.compute.v2 import server as _server
+from openstack.compute.v2 import server_diagnostics as _server_diagnostics
 from openstack.compute.v2 import server_group as _server_group
 from openstack.compute.v2 import server_interface as _server_interface
 from openstack.compute.v2 import server_ip
@@ -50,7 +52,7 @@ class Proxy(proxy.Proxy):
         :returns: A generator of extension instances.
         :rtype: :class:`~openstack.compute.v2.extension.Extension`
         """
-        return self._list(extension.Extension, paginated=True)
+        return self._list(extension.Extension)
 
     def find_flavor(self, name_or_id, ignore_missing=True):
         """Find a single flavor
@@ -112,13 +114,121 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.compute.v2.flavor.FlavorDetail` objects,
             otherwise :class:`~openstack.compute.v2.flavor.Flavor`.
             *Default: ``True``*
-        :param kwargs \*\*query: Optional query parameters to be sent to limit
+        :param kwargs query: Optional query parameters to be sent to limit
                                  the flavors being returned.
 
         :returns: A generator of flavor objects
         """
         flv = _flavor.FlavorDetail if details else _flavor.Flavor
-        return self._list(flv, paginated=True, **query)
+        return self._list(flv, **query)
+
+    def aggregates(self, **query):
+        """Return a generator of aggregate
+
+        :param kwargs query: Optional query parameters to be sent to limit
+            the aggregates being returned.
+
+        :returns: A generator of aggregate
+        :rtype: class: `~openstack.compute.v2.aggregate.Aggregate`
+        """
+        return self._list(_aggregate.Aggregate, **query)
+
+    def get_aggregate(self, aggregate):
+        """Get a single host aggregate
+
+        :param image: The value can be the ID of an aggregate or a
+                      :class:`~openstack.compute.v2.aggregate.Aggregate`
+                      instance.
+
+        :returns: One :class:`~openstack.compute.v2.aggregate.Aggregate`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+                 when no resource can be found.
+        """
+        return self._get(_aggregate.Aggregate, aggregate)
+
+    def create_aggregate(self, **attrs):
+        """Create a new host aggregate from attributes
+
+        :param dict attrs: Keyword arguments which will be used to create a
+                           :class:`~openstack.compute.v2.aggregate.Aggregate`,
+                           comprised of the properties on the Aggregate class.
+
+        :returns: The results of aggregate creation
+        :rtype: :class:`~openstack.compute.v2.aggregate.Aggregate`
+        """
+        return self._create(_aggregate.Aggregate, **attrs)
+
+    def update_aggregate(self, aggregate, **attrs):
+        """Update a host aggregate
+
+        :param server: Either the ID of a host aggregate or a
+                       :class:`~openstack.compute.v2.aggregate.Aggregate`
+                       instance.
+        :attrs kwargs: The attributes to update on the aggregate represented
+                       by ``aggregate``.
+
+        :returns: The updated aggregate
+        :rtype: :class:`~openstack.compute.v2.aggregate.Aggregate`
+        """
+        return self._update(_aggregate.Aggregate, aggregate, **attrs)
+
+    def delete_aggregate(self, aggregate, ignore_missing=True):
+        """Delete a host aggregate
+
+        :param keypair: The value can be either the ID of an aggregate or a
+                        :class:`~openstack.compute.v2.aggregate.Aggregate`
+                        instance.
+        :param bool ignore_missing: When set to ``False``
+                    :class:`~openstack.exceptions.ResourceNotFound` will be
+                    raised when the aggregate does not exist.
+                    When set to ``True``, no exception will be set when
+                    attempting to delete a nonexistent aggregate.
+
+        :returns: ``None``
+        """
+        self._delete(_aggregate.Aggregate, aggregate,
+                     ignore_missing=ignore_missing)
+
+    def add_host_to_aggregate(self, aggregate, host):
+        """Adds a host to an aggregate
+
+        :param aggregate: Either the ID of a aggregate or a
+                    :class:`~openstack.compute.v2.aggregate.Aggregate`
+                    instance.
+        :param str host: The host to add to the aggregate
+
+        :returns: One :class:`~openstack.compute.v2.aggregate.Aggregate`
+        """
+        aggregate = self._get_resource(_aggregate.Aggregate, aggregate)
+        return aggregate.add_host(self, host)
+
+    def remove_host_from_aggregate(self, aggregate, host):
+        """Removes a host from an aggregate
+
+        :param aggregate: Either the ID of a aggregate or a
+                    :class:`~openstack.compute.v2.aggregate.Aggregate`
+                    instance.
+        :param str host: The host to remove from the aggregate
+
+        :returns: One :class:`~openstack.compute.v2.aggregate.Aggregate`
+        """
+        aggregate = self._get_resource(_aggregate.Aggregate, aggregate)
+        return aggregate.remove_host(self, host)
+
+    def set_aggregate_metadata(self, aggregate, metadata):
+        """Creates or replaces metadata for an aggregate
+
+        :param aggregate: Either the ID of a aggregate or a
+                    :class:`~openstack.compute.v2.aggregate.Aggregate`
+                    instance.
+        :param dict metadata: Metadata key and value pairs. The maximum
+                    size for each metadata key and value pair
+                    is 255 bytes.
+
+        :returns: One :class:`~openstack.compute.v2.aggregate.Aggregate`
+        """
+        aggregate = self._get_resource(_aggregate.Aggregate, aggregate)
+        return aggregate.set_metadata(self, metadata)
 
     def delete_image(self, image, ignore_missing=True):
         """Delete an image
@@ -168,13 +278,13 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.compute.v2.image.ImageDetail` objects,
             otherwise :class:`~openstack.compute.v2.image.Image`.
             *Default: ``True``*
-        :param kwargs \*\*query: Optional query parameters to be sent to limit
+        :param kwargs query: Optional query parameters to be sent to limit
                                  the resources being returned.
 
         :returns: A generator of image objects
         """
         img = _image.ImageDetail if details else _image.Image
-        return self._list(img, paginated=True, **query)
+        return self._list(img, **query)
 
     def _get_base_resource(self, res, base):
         # Metadata calls for Image and Server can work for both those
@@ -300,7 +410,7 @@ class Proxy(proxy.Proxy):
         :returns: A generator of keypair objects
         :rtype: :class:`~openstack.compute.v2.keypair.Keypair`
         """
-        return self._list(_keypair.Keypair, paginated=False)
+        return self._list(_keypair.Keypair)
 
     def get_limits(self):
         """Retrieve limits that are applied to the project's account
@@ -371,45 +481,22 @@ class Proxy(proxy.Proxy):
         """
         return self._get(_server.Server, server)
 
-    def servers(self, details=True, **query):
+    def servers(self, details=True, all_projects=False, **query):
         """Retrieve a generator of servers
 
         :param bool details: When set to ``False``
-                    :class:`~openstack.compute.v2.server.Server` instances
-                    will be returned. The default, ``True``, will cause
-                    :class:`~openstack.compute.v2.server.ServerDetail`
-                    instances to be returned.
-        :param kwargs \*\*query: Optional query parameters to be sent to limit
-            the servers being returned.  Available parameters include:
-
-            * changes_since: A time/date stamp for when the server last changed
-                             status.
-            * image: An image resource or ID.
-            * flavor: A flavor resource or ID.
-            * name: Name of the server as a string.  Can be queried with
-                    regular expressions. The regular expression
-                    ?name=bob returns both bob and bobb.  If you must match on
-                    only bob, you can use a regular expression that
-                    matches the syntax of the underlying database server that
-                    is implemented for Compute, such as MySQL or PostgreSQL.
-            * status: Value of the status of the server so that you can filter
-                    on "ACTIVE" for example.
-            * host: Name of the host as a string.
-            * limit: Requests a specified page size of returned items from the
-                     query.  Returns a number of items up to the specified
-                     limit value. Use the limit parameter to make an initial
-                     limited request and use the ID of the last-seen item from
-                     the response as the marker parameter value in a subsequent
-                     limited request.
-            * marker: Specifies the ID of the last-seen item. Use the limit
-                      parameter to make an initial limited request and use the
-                      ID of the last-seen item from the response as the marker
-                      parameter value in a subsequent limited request.
+            instances with only basic data will be returned. The default,
+            ``True``, will cause instances with full data to be returned.
+        :param kwargs query: Optional query parameters to be sent to limit
+            the servers being returned. Available parameters can be seen
+            under https://developer.openstack.org/api-ref/compute/#list-servers
 
         :returns: A generator of server instances.
         """
-        srv = _server.ServerDetail if details else _server.Server
-        return self._list(srv, paginated=True, **query)
+        if all_projects:
+            query['all_projects'] = True
+        base_path = '/servers/detail' if details else None
+        return self._list(_server.Server, base_path=base_path, **query)
 
     def update_server(self, server, **attrs):
         """Update a server
@@ -819,16 +906,20 @@ class Proxy(proxy.Proxy):
                         interval=2, wait=120):
         """Wait for a server to be in a particular status.
 
-        :param res: The resource to wait on to reach the specified status.
-                    The resource must have a ``status`` attribute.
-        :type resource: A :class:`~openstack.resource.Resource` object.
+        :param server:
+            The :class:`~openstack.compute.v2.server.Server` to wait on
+            to reach the specified status.
+        :type server: :class:`~openstack.compute.v2.server.Server`:
         :param status: Desired status.
-        :param failures: Statuses that would be interpreted as failures.
+        :param failures:
+            Statuses that would be interpreted as failures.
         :type failures: :py:class:`list`
-        :param interval: Number of seconds to wait before to consecutive
-                         checks. Default to 2.
-        :param wait: Maximum number of seconds to wait before the change.
-                     Default to 120.
+        :param int interval:
+            Number of seconds to wait before to consecutive checks.
+            Default to 2.
+        :param int wait:
+            Maximum number of seconds to wait before the change.
+            Default to 120.
         :returns: The resource is returned on success.
         :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
                  to the desired status failed to occur in specified seconds.
@@ -921,7 +1012,7 @@ class Proxy(proxy.Proxy):
         :rtype: :class:`~openstack.compute.v2.server_interface.ServerInterface`
         """
         server_id = resource.Resource._get_id(server)
-        return self._list(_server_interface.ServerInterface, paginated=False,
+        return self._list(_server_interface.ServerInterface,
                           server_id=server_id)
 
     def server_ips(self, server, network_label=None):
@@ -936,7 +1027,7 @@ class Proxy(proxy.Proxy):
         :rtype: :class:`~openstack.compute.v2.server_ip.ServerIP`
         """
         server_id = resource.Resource._get_id(server)
-        return self._list(server_ip.ServerIP, paginated=False,
+        return self._list(server_ip.ServerIP,
                           server_id=server_id, network_label=network_label)
 
     def availability_zones(self, details=False):
@@ -950,12 +1041,11 @@ class Proxy(proxy.Proxy):
         :rtype: :class:`~openstack.compute.v2.availability_zone.\
                         AvailabilityZone`
         """
-        if details:
-            az = availability_zone.AvailabilityZoneDetail
-        else:
-            az = availability_zone.AvailabilityZone
+        base_path = '/os-availability-zone/detail' if details else None
 
-        return self._list(az, paginated=False)
+        return self._list(
+            availability_zone.AvailabilityZone,
+            base_path=base_path)
 
     def get_server_metadata(self, server):
         """Return a dictionary of metadata for a server
@@ -1073,13 +1163,13 @@ class Proxy(proxy.Proxy):
     def server_groups(self, **query):
         """Return a generator of server groups
 
-        :param kwargs \*\*query: Optional query parameters to be sent to limit
+        :param kwargs query: Optional query parameters to be sent to limit
                                  the resources being returned.
 
         :returns: A generator of ServerGroup objects
         :rtype: :class:`~openstack.compute.v2.server_group.ServerGroup`
         """
-        return self._list(_server_group.ServerGroup, paginated=False, **query)
+        return self._list(_server_group.ServerGroup, **query)
 
     def hypervisors(self, details=False):
         """Return a generator of hypervisor
@@ -1097,7 +1187,7 @@ class Proxy(proxy.Proxy):
         else:
             hypervisor = _hypervisor.Hypervisor
 
-        return self._list(hypervisor, paginated=False)
+        return self._list(hypervisor)
 
     def find_hypervisor(self, name_or_id, ignore_missing=True):
         """Find a hypervisor from name or id to get the corresponding info
@@ -1176,7 +1266,7 @@ class Proxy(proxy.Proxy):
         :rtype: class: `~openstack.compute.v2.service.Service`
         """
 
-        return self._list(_service.Service, paginated=False)
+        return self._list(_service.Service)
 
     def create_volume_attachment(self, server, **attrs):
         """Create a new volume attachment from attributes
@@ -1298,7 +1388,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.compute.v2.volume_attachment.VolumeAttachment`
         """
         server_id = resource.Resource._get_id(server)
-        return self._list(_volume_attachment.VolumeAttachment, paginated=False,
+        return self._list(_volume_attachment.VolumeAttachment,
                           server_id=server_id)
 
     def migrate_server(self, server):
@@ -1354,3 +1444,20 @@ class Proxy(proxy.Proxy):
                  to delete failed to occur in the specified seconds.
         """
         return resource.wait_for_delete(self, res, interval, wait)
+
+    def get_server_diagnostics(self, server):
+        """Get a single server diagnostics
+
+        :param server: This parameter need to be specified when ServerInterface
+                       ID is given as value. It can be either the ID of a
+                       server or a :class:`~openstack.compute.v2.server.Server`
+                       instance that the interface belongs to.
+
+        :returns: One
+            :class:`~openstack.compute.v2.server_diagnostics.ServerDiagnostics`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+                 when no resource can be found.
+        """
+        server_id = self._get_resource(_server.Server, server).id
+        return self._get(_server_diagnostics.ServerDiagnostics,
+                         server_id=server_id, requires_id=False)
